@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/*$Id: gdevbbox.c 9852 2009-07-09 05:59:44Z henrys $ */
+/*$Id: gdevbbox.c 10620 2010-01-19 15:48:57Z robin $ */
 /* Device for tracking bounding box */
 #include "math_.h"
 #include "memory_.h"
@@ -144,7 +144,9 @@ gx_device_bbox gs_bbox_device =
      NULL,			/* fill_linear_color_triangle */
      NULL,			/* update_spot_equivalent_colors */
      NULL,			/* ret_devn_params */
-     bbox_fillpage			/* fillpage */
+     bbox_fillpage,		/* fillpage */
+     NULL,                      /* push_transparency_state */
+     NULL                       /* pop_transparency_state */
     },
     0,				/* target */
     1,				/*true *//* free_standing */
@@ -234,7 +236,7 @@ bbox_copy_params(gx_device_bbox * bdev, bool remap_colors)
 }
 
 #define GX_DC_IS_TRANSPARENT(pdevc, bdev)\
-  (gx_dc_pure_color(pdevc) == (bdev)->transparent && gx_dc_is_pure(pdevc))
+  (gx_dc_is_pure(pdevc) && gx_dc_pure_color(pdevc) == (bdev)->transparent)
 
 static int
 bbox_close_device(gx_device * dev)
@@ -704,7 +706,8 @@ bbox_fill_triangle(gx_device * dev,
 static int
 bbox_draw_thin_line(gx_device * dev,
 		    fixed fx0, fixed fy0, fixed fx1, fixed fy1,
-		    const gx_device_color * pdevc, gs_logical_operation_t lop)
+		    const gx_device_color * pdevc, gs_logical_operation_t lop,
+		    fixed adjustx, fixed adjusty)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
     /* Skip the call if there is no target. */
@@ -712,7 +715,7 @@ bbox_draw_thin_line(gx_device * dev,
     int code =
 	(tdev == 0 ? 0 :
 	 dev_proc(tdev, draw_thin_line)
-	 (tdev, fx0, fy0, fx1, fy0, pdevc, lop));
+	 (tdev, fx0, fy0, fx1, fy0, pdevc, lop, adjustx, adjusty));
 
     if (!GX_DC_IS_TRANSPARENT(pdevc, bdev)) {
 	fixed xmin, ymin, xmax, ymax;
