@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevdflt.c 9288 2008-12-13 20:05:37Z leonardo $ */
+/* $Id: gdevdflt.c 10200 2009-10-21 03:50:46Z mvrhel $ */
 /* Default device implementation */
 #include "math_.h"
 #include "gx.h"
@@ -886,7 +886,8 @@ gx_default_composite_clist_read_update(gs_composite_t *pxcte, gx_device * cdev,
  * Default handler for get_cropping returns no cropping.
  */
 int
-gx_default_composite_get_cropping(const gs_composite_t *pxcte, int *ry, int *rheight)
+gx_default_composite_get_cropping(const gs_composite_t *pxcte, int *ry, int *rheight,
+                                  int cropping_min, int cropping_max)
 {
     return 0;			/* No cropping. */
 }
