@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevnfwd.c 9288 2008-12-13 20:05:37Z leonardo $ */
+/* $Id: gdevnfwd.c 10620 2010-01-19 15:48:57Z robin $ */
 /* Null and forwarding device implementation */
 #include "gx.h"
 #include "gserrors.h"
@@ -128,9 +128,12 @@ gx_forward_close_device(gx_device * dev)
 {
     gx_device_forward * const fdev = (gx_device_forward *)dev;
     gx_device *tdev = fdev->target;
-
-    return (tdev == 0) ? gx_default_close_device(dev)
+    int code = (tdev == 0) ? gx_default_close_device(dev)
 		       : dev_proc(tdev, close_device)(tdev);
+
+    if (tdev)
+	tdev->is_open = false;		/* flag corresponds to the state */
+    return code;
 }
 
 void
@@ -284,11 +287,16 @@ gx_forward_put_params(gx_device * dev, gs_param_list * plist)
 {
     gx_device_forward * const fdev = (gx_device_forward *)dev;
     gx_device *tdev = fdev->target;
+    bool was_open;
     int code;
 
     if (tdev == 0)
 	return gx_default_put_params(dev, plist);
+    was_open = tdev->is_open;
     code = dev_proc(tdev, put_params)(tdev, plist);
+    if (code == 0 && !tdev->is_open) {
+	    code = was_open ? 1 : 0;   /* target device closed */
+    }
     if (code >= 0)
 	gx_device_decache_colors(dev);
     return code;
@@ -476,7 +484,8 @@ gx_forward_fill_triangle(gx_device * dev,
 int
 gx_forward_draw_thin_line(gx_device * dev,
 			  fixed fx0, fixed fy0, fixed fx1, fixed fy1,
-	       const gx_drawing_color * pdcolor, gs_logical_operation_t lop)
+	       const gx_drawing_color * pdcolor, gs_logical_operation_t lop,
+			  fixed adjustx, fixed adjusty)
 {
     gx_device_forward * const fdev = (gx_device_forward *)dev;
     gx_device *tdev = fdev->target;
@@ -484,7 +493,7 @@ gx_forward_draw_thin_line(gx_device * dev,
 	(tdev == 0 ? (tdev = dev, gx_default_draw_thin_line) :
 	 dev_proc(tdev, draw_thin_line));
 
-    return proc(tdev, fx0, fy0, fx1, fy1, pdcolor, lop);
+    return proc(tdev, fx0, fy0, fx1, fy1, pdcolor, lop, adjustx, adjusty);
 }
 
 int
@@ -1086,7 +1095,8 @@ static int
 null_draw_thin_line(gx_device * dev,
 		    fixed fx0, fixed fy0, fixed fx1, fixed fy1,
 		    const gx_drawing_color * pdcolor,
-		    gs_logical_operation_t lop)
+		    gs_logical_operation_t lop,
+		    fixed adjustx, fixed adjusty)
 {
     return 0;
 }
