@@ -10,13 +10,16 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevp14.h 9880 2009-07-22 21:09:42Z mvrhel $ */
+/* $Id: gdevp14.h 10624 2010-01-20 18:11:39Z mvrhel $ */
 /* Definitions and interface for PDF 1.4 rendering device */
 
 #ifndef gdevp14_INCLUDED
 #  define gdevp14_INCLUDED
 
 #include "gxcmap.h"
+#include "gsmatrix.h"
+#include "gxcolor2.h"
+#include "gxdcolor.h"
 #include "gxpcolor.h"
 #include "gdevdevn.h"
 
@@ -56,8 +59,38 @@ typedef struct {
 		    gs_imager_state * pis, gx_device * target);
 } pdf14_procs_s;
 
+
 typedef pdf14_procs_s pdf14_procs_t;
 
+/* A stack structure for the softmask buffers. 
+   The mask will be pdf14 buffers that are wrapped
+   in a refernce counted structure.  We need this to
+   be referenced counted since we need to be able to push
+   multiple copies of the same buffer on the
+   stack as we get multiple q operations when
+   a soft mask is present in the graphic state. */
+
+typedef struct pdf14_rcmask_s pdf14_rcmask_t;
+
+struct pdf14_rcmask_s {
+
+    pdf14_buf   *mask_buf;
+    rc_header rc;
+    gs_memory_t *memory;
+     
+};
+
+typedef struct pdf14_mask_s pdf14_mask_t;
+
+struct pdf14_mask_s {
+
+    pdf14_rcmask_t *rc_mask;
+    pdf14_mask_t *previous;
+    gs_memory_t *memory;
+ 
+};
+
+
 /* A structure to hold information
  * about the parent color related
  * procs and other information.
@@ -74,6 +107,8 @@ struct pdf14_parent_color_s {
     int num_components;
     bool isadditive;
     gx_color_polarity_t polarity;
+    byte comp_shift[GX_DEVICE_COLOR_MAX_COMPONENTS]; /* These are needed for the shading code */
+    byte comp_bits[GX_DEVICE_COLOR_MAX_COMPONENTS];
     byte depth;  /* used in clist writer cmd_put_color */
     const gx_color_map_procs *(*get_cmap_procs)(const gs_imager_state *,
 						     const gx_device *);
@@ -112,7 +147,7 @@ struct pdf14_buf_s {
     byte *data;
     byte *transfer_fn;
     gs_int_rect bbox;
-    pdf14_buf *maskbuf; /* Save pdf14_ctx_s::maksbuf. */
+    pdf14_mask_t *maskbuf;
     bool idle;
 
     bool SMask_is_CIE;
@@ -126,7 +161,7 @@ struct pdf14_buf_s {
 
 struct pdf14_ctx_s {
     pdf14_buf *stack;
-    pdf14_buf *maskbuf;
+    pdf14_mask_t *maskbuf;
     gs_memory_t *memory;
     gs_int_rect rect;
     bool additive;
@@ -233,16 +268,6 @@ pdf14_put_devn_params(gx_device * pdev, gs_devn_params * pdevn_params,
 int pdf14_get_buffer_information(const gx_device * dev, gx_pattern_trans_t *transbuff);
 
 /* Not static due to call from pattern logic */
-int
-pdf14_disable_device(gx_device * dev);
-
-
-/* Used for filling rects when we are doing a fill with a pattern that
-   has transparency */
-static int
-pdf14_tile_pattern_fill(gx_device * pdev, const gs_imager_state * pis,
-		     gx_path * ppath, const gx_fill_params * params,
-		 const gx_device_color * pdevc, const gx_clip_path * pcpath);
-
+int pdf14_disable_device(gx_device * dev);
 
 #endif /* gdevp14_INCLUDED */
