@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfb.c 9810 2009-06-22 07:41:01Z ken $ */
+/* $Id: gdevpdfb.c 9985 2009-08-13 07:57:02Z ken $ */
 /* Low-level bitmap image handling for PDF-writing driver */
 #include "string_.h"
 #include "gx.h"
@@ -153,7 +153,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	if (one == gx_no_color_index)
 	    return 0;
 	/* If a mask has an id, assume it's a character. */
-	if (id != gx_no_bitmap_id && sourcex == 0) {
+	if (id != gx_no_bitmap_id && sourcex == 0 && show_enum) {
 	    pdf_char_proc_t *pcp;
 
 	    if (show_enum->use_wxy_float)
