@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfg.c 9669 2009-04-21 15:01:01Z ken $ */
+/* $Id: gdevpdfg.c 10445 2009-12-04 00:33:04Z robin $ */
 /* Graphics state management for pdfwrite driver */
 #include "math_.h"
 #include "string_.h"
@@ -165,7 +165,9 @@ pdf_viewer_state_from_imager_state_aux(pdf_viewer_state *pvs, const gs_imager_st
     pvs->stroke_overprint = false;
     pvs->stroke_adjust = false;
     pvs->line_params.half_width = 0.5;
-    pvs->line_params.cap = 0;
+    pvs->line_params.start_cap = 0;
+    pvs->line_params.end_cap = 0;
+    pvs->line_params.dash_cap = 0;
     pvs->line_params.join = 0;
     pvs->line_params.curve_join = 0;
     pvs->line_params.miter_limit = 10.0;
@@ -825,7 +827,12 @@ pdf_write_spot_function(gx_device_pdf *pdev, const gx_ht_order *porder,
      * simplicity, we always use 16.
      */
     if (num_bits > 0x10000)
-	return_error(gs_error_rangecheck);
+	/* rangecheck is a 'special case' in gdev_pdf_fill_path, if this error is encountered
+	 * then it 'falls back' to a different method assuming its handling transparency in an
+	 * old PDF output version. But if we fail to write the halftone, we want to abort
+	 * so use limitcheck instead.
+	 */
+	return_error(gs_error_limitcheck);
     params.BitsPerSample = 16;
     params.Encode = 0;
     /*
@@ -918,7 +925,9 @@ pdf_write_spot_halftone(gx_device_pdf *pdev, const gs_spot_halftone *psht,
  notrec:
     if (i == countof(ht_functions)) {
 	/* Create and write a Function for the spot function. */
-	pdf_write_spot_function(pdev, porder, &spot_id);
+	code = pdf_write_spot_function(pdev, porder, &spot_id);
+	if (code < 0) 
+	    return code;
     }	
     *pid = id = pdf_begin_separate(pdev);
     s = pdev->strm;
@@ -1268,6 +1277,7 @@ pdf_end_gstate(gx_device_pdf *pdev, pdf_resource_t *pres)
 	
 	if (code < 0)
 	    return code;
+	pres->where_used |= pdev->used_mask;
 	code = pdf_open_page(pdev, PDF_IN_STREAM);
 	if (code < 0)
 	    return code;
