@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpsft.c 9651 2009-04-20 06:56:10Z ken $ */
+/* $Id: gdevpsft.c 10255 2009-11-02 05:21:28Z marcos $ */
 /* Write an embedded TrueType font */
 #include "memory_.h"
 #include <stdlib.h>		/* for qsort */
@@ -304,7 +304,8 @@ write_cmap_0(stream *s, byte* entries /*[CMAP_ENTRIES_SIZE]*/, uint num_glyphs)
 {
     int i;
 
-    memset(entries + 2 * num_glyphs, 0, CMAP_ENTRIES_SIZE - 2 * num_glyphs);
+    if (CMAP_ENTRIES_SIZE - 2 * num_glyphs>0)
+      memset(entries + 2 * num_glyphs, 0, CMAP_ENTRIES_SIZE - 2 * num_glyphs);
     stream_write(s, cmap_initial_0, sizeof(cmap_initial_0));
     for (i = 0; i <= 0xff; ++i)
 	sputc(s, (byte)entries[2 * i + 1]);
@@ -719,7 +720,7 @@ static inline bool check_position(int pos1, int pos2)
     return true;
 }
 
-void remove_table(byte *tables, char *tag, uint *numTables)
+static void remove_table(byte *tables, char *tag, uint *numTables)
 {
     /* Not a high performance implementation because it is called seldom. */
     int i;
@@ -945,8 +946,8 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 		return_error(gs_error_unregistered);
 	    }
 	    loca_length = (enlarged_numGlyphs + 1) << 2;
-	    remove_table(tables, "hdmx", &numTables);
-	    remove_table(tables, "vdmx", &numTables);
+	    remove_table(tables, (char *)"hdmx", &numTables);
+	    remove_table(tables, (char *)"vdmx", &numTables);
 	} else
 	    loca_length = (numGlyphs + 1) << 2;
 	indexToLocFormat = (glyf_length > 0x1fffc);
