@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevwts.c 9784 2009-06-09 03:32:01Z henrys $ */
+/* $Id: gdevwts.c 10078 2009-09-21 19:38:30Z ray $ */
 /* ALPHA: Sample Device that provides WTS screening and IMDI color management */
 /* TODO: this should be configurable */
 #define LINK_ICC_NAME	"link.icc"
@@ -342,6 +342,14 @@ wts_init_halftones(gx_device_wts *wdev, int n_planes)
 	    char wts_fn[256];
 
 	    sprintf(wts_fn, "wts_plane_%d", i);
+            {
+              FILE *f;
+              if ((f=fopen(wts_fn,"r"))) {
+                fclose(f);
+              } else {
+	        sprintf(wts_fn, "/usr/local/lib/ghostscript/wts_plane_%d", i);
+              }
+            }
 	    code = wts_load_halftone(wdev->memory, &wdev->wcooked[i], wts_fn);
 	    if (code < 0)
 		return gs_throw1(code, "could not open file '%s'", wts_fn);
@@ -465,6 +473,7 @@ wtsimdi_open_device(gx_device *dev)
     icc *icco;
     icmLuBase *luo;
     imdi *mdo;
+    char link_icc_name[256];
 
     /*
      * We replace create_buf_device so we can replace copy_alpha 
@@ -474,9 +483,19 @@ wtsimdi_open_device(gx_device *dev)
 	wtsimdi_create_buf_device;
     /* Open and read profile */
 
-    fp = new_icmFileStd_name((char *)LINK_ICC_NAME, (char *)"rb");
+    sprintf(link_icc_name, "%s", LINK_ICC_NAME);
+    {
+      FILE *f;
+      if ((f=fopen(link_icc_name,"r"))) {
+        fclose(f);
+      } else {
+        sprintf(link_icc_name, "/usr/local/lib/ghostscript/%s", LINK_ICC_NAME);
+      }
+    }
+
+    fp = new_icmFileStd_name((char *)link_icc_name, (char *)"rb");
     if (!fp)
-	return gs_throw1(-1, "could not open file '%s'", LINK_ICC_NAME);
+	return gs_throw1(-1, "could not open file '%s'", link_icc_name);
 
     icco = new_icc();
     if (!icco)
@@ -968,7 +987,6 @@ wtsimdi_create_buf_device(gx_device **pbdev, gx_device *target, int y,
 	   implemented or replaced with a default implementation. The following
 	   three don't have significant usage in testing with Altona.
 	*/
-	set_dev_proc(*pbdev, strip_copy_rop, gx_default_strip_copy_rop);
 	set_dev_proc(*pbdev, copy_alpha, gx_default_copy_alpha);
 	set_dev_proc(*pbdev, copy_color, gx_default_copy_color);
     }
