@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gscolor2.c 9567 2009-03-19 01:48:02Z henrys $ */
+/* $Id: gscolor2.c 10648 2010-01-27 14:55:55Z robin $ */
 /* Level 2 color operators for Ghostscript library */
 #include "memory_.h"
 #include "gx.h"
@@ -95,6 +95,13 @@ gs_currentcolor(const gs_state * pgs)
     return pgs->ccolor;
 }
 
+/* currentdevicecolor */
+const gx_device_color *
+gs_currentdevicecolor(const gs_state * pgs)
+{
+    return pgs->dev_color;
+}
+
 /* ------ Internal procedures ------ */
 
 /* GC descriptors */
