@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsfont.c 9466 2009-02-10 17:38:02Z marcos $ */
+/* $Id: gsfont.c 10458 2009-12-07 05:46:36Z alexcher $ */
 /* Font operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -1029,7 +1029,7 @@ gs_default_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 	    info->v.x = sbw[0];
 	    info->v.y = sbw[1];
 	} else {
-	    gs_distance_transform(sbw[0], sbw[1], pmat, &info->v);
+	    code = gs_distance_transform(sbw[0], sbw[1], pmat, &info->v);
 	    if (code < 0)
 		return code;
 	}
