@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsmalloc.c 8721 2008-05-09 02:18:14Z ray $ */
+/* $Id: gsmalloc.c 10005 2009-08-18 20:41:17Z ray $ */
 /* C heap allocator */
 #include "malloc_.h"
 #include "gdebug.h"
@@ -209,8 +209,8 @@ gs_heap_alloc_bytes(gs_memory_t * mem, uint size, client_name_t cname)
 	gs_alloc_fill(ptr, gs_alloc_fill_alloc, size);
 #ifdef DEBUG
     if (gs_debug_c('a') || msg != ok_msg)
-	dlprintf4("[a+]gs_malloc(%s)(%u) = 0x%lx: %s\n",
-		  client_name_string(cname), size, (ulong) ptr, msg);
+	dlprintf6("[a+]gs_malloc(%s)(%u) = 0x%lx: %s, used=%ld, max=%ld\n",
+		  client_name_string(cname), size, (ulong) ptr, msg, mmem->used, mmem->max_used);
 #endif
     return ptr;
 #undef set_msg
