@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsptype1.c 9880 2009-07-22 21:09:42Z mvrhel $ */
+/* $Id: gsptype1.c 10534 2009-12-20 06:52:50Z alexcher $ */
 /* PatternType 1 pattern implementation */
 #include "math_.h"
 #include "memory_.h"
@@ -735,8 +735,12 @@ image_PaintProc(const gs_client_color * pcolor, gs_state * pgs)
 	pcspace = gs_cspace_new_DeviceGray(pgs->memory);
     } else
         pcspace = ppmap->pcspace;
-    gs_gsave(pgs);
-    gs_setcolorspace(pgs, pcspace);
+    code = gs_gsave(pgs);
+    if (code < 0)
+        return code;
+    code = gs_setcolorspace(pgs, pcspace);
+    if (code < 0)
+        return code;
     if (transparent)
         gs_image4_t_init( (gs_image4_t *) &image, pcspace);
     else
