@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsstate.c 9892 2009-07-27 00:30:35Z marcos $ */
+/* $Id: gsstate.c 10557 2009-12-26 18:32:35Z alexcher $ */
 /* Miscellaneous graphics state operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -491,11 +491,11 @@ gs_state_copy(gs_state * pgs, gs_memory_t * mem)
 
     pgs->view_clip = 0;
     pnew = gstate_clone(pgs, mem, "gs_gstate", copy_for_gstate);
+    if (pnew == 0)
+	return 0;
     clip_stack_rc_adjust(pnew->clip_stack, 1, "gs_state_copy");
     rc_increment(pnew->dfilter_stack);
     pgs->view_clip = view_clip;
-    if (pnew == 0)
-	return 0;
     pnew->saved = 0;
     /*
      * Prevent dangling references from the show_gstate pointer.  If
@@ -741,7 +741,9 @@ gs_initgraphics(gs_state * pgs)
     if ((code = gs_newpath(pgs)) < 0 ||
 	(code = gs_initclip(pgs)) < 0 ||
 	(code = gs_setlinewidth(pgs, 1.0)) < 0 ||
-	(code = gs_setlinecap(pgs, gstate_initial.line_params.cap)) < 0 ||
+	(code = gs_setlinestartcap(pgs, gstate_initial.line_params.start_cap)) < 0 ||
+	(code = gs_setlineendcap(pgs, gstate_initial.line_params.end_cap)) < 0 ||
+	(code = gs_setlinedashcap(pgs, gstate_initial.line_params.dash_cap)) < 0 ||
 	(code = gs_setlinejoin(pgs, gstate_initial.line_params.join)) < 0 ||
 	(code = gs_setcurvejoin(pgs, gstate_initial.line_params.curve_join)) < 0 ||
 	(code = gs_setdash(pgs, (float *)0, 0, 0.0)) < 0 ||
