@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gstrans.c 9715 2009-04-30 18:34:49Z mvrhel $ */
+/* $Id: gstrans.c 10414 2009-11-30 23:37:23Z mvrhel $ */
 /* Implementation of transparency, other than rendering */
 #include "math_.h"
 #include "memory_.h"
@@ -363,6 +363,110 @@ gx_end_transparency_group(gs_imager_state * pis, gx_device * pdev)
 	return 0;
 }
 
+
+/* Commands for handling q softmask Q in graphic states */
+
+int
+gs_push_transparency_state(gs_state *pgs)
+{
+    gs_pdf14trans_params_t params = { 0 };
+    gs_imager_state * pis = (gs_imager_state *)pgs;
+    int code;
+
+    /* Set the pending flag to true, which indicates
+       that we need to watch for end transparency 
+       soft masks when we are at this graphic state
+       level */
+
+    /* pis->trans_flags.xstate_pending = true; */
+
+    /* Actually I believe the above flag is not 
+       needed.  We really should be watching for
+       the softmask even at the base level.  What
+       we need to watch for are q operations after
+       a soft mask end has occured. */
+
+    /* Check if we have a change flag set to true.
+       this indicates that a softmask is present.
+       We will need to send a push state to save
+       the current soft mask, so that we can
+       restore it later */
+
+    if (pis->trans_flags.xstate_change) {
+
+        if_debug0('v', "[v]gs_push_transparency_state sending\n");
+        params.pdf14_op = PDF14_PUSH_TRANS_STATE;  
+        code = gs_state_update_pdf14trans(pgs, &params);
+        if (code < 0) 
+            return(code);
+
+    } else {
+
+        if_debug0('v', "[v]gs_push_transparency_state NOT sending\n");
+
+    }
+
+
+    return(0);
+}
+
+
+int
+gs_pop_transparency_state(gs_state *pgs)
+{
+    gs_pdf14trans_params_t params = { 0 };
+    gs_imager_state * pis = (gs_imager_state *)pgs;
+    int code;
+
+    /* Check if flag is set, which indicates that we have 
+       an active softmask for the graphic state.  We
+       need to communicate to the compositor to pop
+       the softmask */
+
+    if ( pis->trans_flags.xstate_change ) {
+    
+        if_debug0('v', "[v]gs_pop_transparency_state sending\n");
+        params.pdf14_op = PDF14_POP_TRANS_STATE;  
+        code = gs_state_update_pdf14trans(pgs, &params);
+        if ( code < 0 )
+            return (code);
+
+    } else {
+
+        if_debug0('v', "[v]gs_pop_transparency_state NOT sending\n");
+
+    }
+
+    /* There is no reason to reset any of the flags since
+       they will be reset by the graphic state restore */
+
+    return(0);
+
+}
+
+
+int
+gx_pop_transparency_state(gs_imager_state * pis, gx_device * pdev)
+{
+    if_debug0('v', "[v]gx_pop_transparency_state\n");
+    if (dev_proc(pdev, pop_transparency_state) != 0)
+	return (*dev_proc(pdev, pop_transparency_state)) (pdev, pis);
+    else
+	return 0;
+}
+
+int
+gx_push_transparency_state(gs_imager_state * pis, gx_device * pdev)
+{
+    if_debug0('v', "[v]gx_push_transparency_state\n");
+    if (dev_proc(pdev, push_transparency_state) != 0)
+	return (*dev_proc(pdev, push_transparency_state)) (pdev, pis);
+    else
+	return 0;
+}
+
+
+
 /*
  * Handler for identity mask transfer functions.
  */
@@ -395,6 +499,7 @@ gs_begin_transparency_mask(gs_state * pgs,
     int i;
     const gs_color_space *blend_color_space;
     gs_imager_state * pis = (gs_imager_state *)pgs;
+    int num_components;
 
     params.pdf14_op = PDF14_BEGIN_TRANS_MASK;
     params.bbox = *pbbox;
@@ -488,48 +593,35 @@ gs_begin_transparency_mask(gs_state * pgs,
 
         }
 
+        /* For the softmask blend color space, we will always use the above blend_color_space. 
+           Problems can occur if we go all the way back to the device color space,
+           which could be DeviceN for a sep device.  Blending to the luminosity
+           channel for this case would not be defined. */
 
-        /* Note that if the /CS parameter was not present in the push 
-        of the transparency group, then we must actually inherent 
-        the previous group color space, or the color space of the
-        target device (process color model).  Note here we just want
-        to set it as a unknown type for clist writing, as we .  We will later 
-        during clist reading 
-        */
+        num_components = cs_num_components(blend_color_space);
+        switch (abs(num_components)) {
 
-        if (ptmp->ColorSpace == NULL) {
+            case 1:				
+                params.group_color = GRAY_SCALE;       
+                params.group_color_numcomps = 1;  /* Need to check */
+                break;
+            case 3:				
+                params.group_color = DEVICE_RGB;       
+                params.group_color_numcomps = 3; 
+                break;
+            case 4:				
+                params.group_color = DEVICE_CMYK;       
+                params.group_color_numcomps = 4; 
+            break;
+            default:
+                /* Transparency soft mask spot
+                   colors are NEVER available. 
+                   We must use the alternate tint
+                   transform */
+            return_error(gs_error_rangecheck);
+            break;
 
-            params.group_color = UNKNOWN;
-            params.group_color_numcomps = 0;
-
-        } else {
-
-
-            switch (cs_num_components(blend_color_space)) {
-
-                case 1:				
-                    params.group_color = GRAY_SCALE;       
-                    params.group_color_numcomps = 1;  /* Need to check */
-                    break;
-                case 3:				
-                    params.group_color = DEVICE_RGB;       
-                    params.group_color_numcomps = 3; 
-                    break;
-                case 4:				
-                    params.group_color = DEVICE_CMYK;       
-                    params.group_color_numcomps = 4; 
-	            break;
-                default:
-                    /* Transparency soft mask spot
-                       colors are NEVER available. 
-                       We must use the alternate tint
-                       transform */
-	            return_error(gs_error_rangecheck);
-	            break;
-
-             }    
-
-        }
+         }    
 
     }
 
@@ -577,6 +669,23 @@ gs_end_transparency_mask(gs_state *pgs,
 			 gs_transparency_channel_selector_t csel)
 {
     gs_pdf14trans_params_t params = { 0 };
+    gs_imager_state * pis = (gs_imager_state *)pgs;
+
+    /* If we have done a q then set a flag to watch for any Qs */
+
+   /* if (pis->trans_flags.xstate_pending)
+        pis->trans_flags.xstate_change = true; */
+
+    /* This should not depend upon if we have encountered a q
+       operation.  We could be setting a softmask, before 
+       there is any q operation.  Unlikely but it could happen.
+       Then if we encouter a q operation (and this flag
+       is true) we will need to 
+       push the mask graphic state (PDF14_PUSH_TRANS_STATE). */
+
+    pis->trans_flags.xstate_change = true;
+
+    if_debug1('v', "[v]xstate_changed set true, gstate level is %d\n", pgs->level);
 
     if_debug2('v', "[v](0x%lx)gs_end_transparency_mask(%d)\n", (ulong)pgs,
 	      (int)csel);
@@ -632,7 +741,20 @@ get_num_pdf14_spot_colors(gs_state * pgs)
      * countspotcolors in lib/pdf_main.ps.
      */
     if (pclist_devn_params != NULL) {
-	return pclist_devn_params->page_spot_colors;
+
+        /* If the sep order names were specified, then we should only allocate
+           for those.  But only the nonstandard colorants that are stored
+           in num_separations.  See devn_put_params for details on this. 
+           Right now, the PDF14 device will always include CMYK.  A future
+           optimization is to be able to NOT have those included in the buffer 
+           allocations if we don't specify them.  It would then be possible to
+           output 8 separations at a time without using compressed color. */
+
+        if (pclist_devn_params->num_separation_order_names == 0)
+	    return pclist_devn_params->page_spot_colors;
+
+        return (pclist_devn_params->separations.num_separations);
+
     }
     return 0;
 }
