@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxdevcli.h 9666 2009-04-20 19:16:24Z mvrhel $ */
+/* $Id: gxdevcli.h 10620 2010-01-19 15:48:57Z robin $ */
 /* Definitions for device clients */
 
 #ifndef gxdevcli_INCLUDED
@@ -1021,10 +1021,16 @@ typedef struct gs_param_list_s gs_param_list;
 #define dev_proc_fill_triangle(proc)\
   dev_t_proc_fill_triangle(proc, gx_device)
 
+		/* adjustx and adjusty were added in 8.71 to get around a
+		 * problem with PCL (Bug 691030). In the fullness of time
+		 * hopefully PCL can be fixed to not need them and they can
+		 * be removed again. */
+
 #define dev_t_proc_draw_thin_line(proc, dev_t)\
   int proc(dev_t *dev,\
     fixed fx0, fixed fy0, fixed fx1, fixed fy1,\
-    const gx_drawing_color *pdcolor, gs_logical_operation_t lop)
+    const gx_drawing_color *pdcolor, gs_logical_operation_t lop,\
+    fixed adjustx, fixed adjusty)
 #define dev_proc_draw_thin_line(proc)\
   dev_t_proc_draw_thin_line(proc, gx_device)
 
@@ -1388,6 +1394,21 @@ typedef struct gs_devn_params_s gs_devn_params;
   int proc(gx_device *dev, gs_imager_state * pis, gx_device_color *pdevc)
 #define dev_proc_fillpage(proc)\
   dev_t_proc_fillpage(proc, gx_device)
+
+
+#define dev_t_proc_push_transparency_state(proc, dev_t)\
+  int proc(gx_device *dev,\
+    gs_imager_state *pis)
+#define dev_proc_push_transparency_state(proc)\
+  dev_t_proc_push_transparency_state(proc, gx_device)
+
+
+#define dev_t_proc_pop_transparency_state(proc, dev_t)\
+  int proc(gx_device *dev,\
+    gs_imager_state *pis)
+#define dev_proc_pop_transparency_state(proc)\
+  dev_t_proc_pop_transparency_state(proc, gx_device)
+
 /* Define the device procedure vector template proper. */
 
 #define gx_device_proc_struct(dev_t)\
@@ -1453,9 +1474,10 @@ typedef struct gs_devn_params_s gs_devn_params;
 	dev_t_proc_update_spot_equivalent_colors((*update_spot_equivalent_colors), dev_t); \
 	dev_t_proc_ret_devn_params((*ret_devn_params), dev_t); \
 	dev_t_proc_fillpage((*fillpage), dev_t); \
+        dev_t_proc_push_transparency_state((*push_transparency_state), dev_t); \
+        dev_t_proc_pop_transparency_state((*pop_transparency_state), dev_t); \
 }
 
-
 /*
  * Provide procedures for passing image data.  image_data and end_image
  * are the equivalents of the obsolete driver procedures.  image_plane_data
