@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxp1fill.c 9880 2009-07-22 21:09:42Z mvrhel $ */
+/* $Id: gxp1fill.c 9931 2009-08-04 01:32:50Z henrys $ */
 /* PatternType 1 filling algorithms */
 #include "string_.h"
 #include "math_.h"
@@ -108,8 +108,7 @@ tile_fill_init(tile_fill_state_t * ptfs, const gx_device_color * pdevc,
 		  m_tile->tmask.rep_height);
     } else
 	px = py = 0;
-    return tile_clip_initialize(&ptfs->cdev, ptfs->tmask, dev, px, py, dev->memory); 
-    /* leak ? was NULL memoryptr */
+    return tile_clip_initialize(&ptfs->cdev, ptfs->tmask, dev, px, py, NULL); 
 }
 
 /*
