@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxttfb.c 9852 2009-07-09 05:59:44Z henrys $ */
+/* $Id: gxttfb.c 10457 2009-12-06 19:12:08Z alexcher $ */
 /* A bridge to True Type interpreter. */
 
 #include "gx.h"
@@ -681,7 +681,7 @@ static int grid_fit(gx_device_spot_analyzer *padev, gx_path *path,
 	code = t1_hinter__sbw(&h.super, sbx, sby, e->w.x, e->w.y);
 	if (code < 0)
 	    return code;
-	gx_path_bbox(path, &bbox);
+	code = gx_path_bbox(path, &bbox);
 	if (code < 0)
 	    return code;
 	memset(&is_stub, 0, sizeof(is_stub));
