@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfapi.c 9760 2009-05-24 10:16:57Z giles $ */
+/* $Id: zfapi.c 10654 2010-01-28 14:44:22Z ken $ */
 /* Font API client */
 
 #include "memory_.h"
@@ -33,6 +33,7 @@
 #include "gspath.h"
 #include "bfont.h"
 #include "dstack.h"
+#include "estack.h"
 #include "ichar.h"
 #include "idict.h"
 #include "iname.h"
@@ -179,10 +180,11 @@ static void sfnts_reader_rstring(sfnts_reader *r, byte *v, int length)
     while (!r->error) {
         int l = min(length, r->length - r->offset);
         memcpy(v, r->p + r->offset, l);
-        r->p += l;
         length -= l;
+	r->offset += l;
         if (length <= 0)
             return;
+	v += l;
         sfnts_next_elem(r);
     }
 }
@@ -509,7 +511,7 @@ static ushort FAPI_FF_get_word(FAPI_font *ff, fapi_font_feature var_id, int inde
 			return 0;
 		    switch (r_btype(&Element)) {
 			case t_name:
-			    name_string_ref(ff->memory, &name, &string);
+			    name_string_ref(ff->memory, &Element, &string);
 			    length += r_size(&string) + 1;
 			    break;
 			case t_real:
@@ -777,15 +779,15 @@ static ushort FAPI_FF_get_subr(FAPI_font *ff, int index, byte *buf, ushort buf_l
     return get_type1_data(ff, &subr, buf, buf_length);
 }
 
-static bool sfnt_get_glyph_offset(ref *pdr, gs_font_type42 *pfont42, int index, ulong *offset0, ulong *offset1)
+static bool sfnt_get_glyph_offset(ref *pdr, gs_font_type42 *pfont42, int index, ulong *offset0)
 {   /* Note : TTC is not supported and probably is unuseful for Type 42. */
     sfnts_reader r;
-    int glyf_elem_size = (2 << pfont42->data.indexToLocFormat);
+    int i, glyf_elem_size = (2 << pfont42->data.indexToLocFormat);
+    ulong off;
 
     sfnts_reader_init(&r, pdr);
     r.seek(&r, pfont42->data.loca + index * glyf_elem_size);
     *offset0 = pfont42->data.glyf + (glyf_elem_size == 2 ? r.rword(&r) * 2 : r.rlong(&r));
-    *offset1 = pfont42->data.glyf + (glyf_elem_size == 2 ? r.rword(&r) * 2 : r.rlong(&r));
     return r.error;
 }
 
@@ -894,10 +896,10 @@ static ushort FAPI_FF_get_glyph(FAPI_font *ff, int char_code, byte *buf, ushort
                 memcpy(buf, data_ptr + mc, min(glyph_length, buf_length)/* safety */);
         } else {
             gs_font_type42 *pfont42 = (gs_font_type42 *)ff->client_font_data;
-            ulong offset0, offset1;
-	    bool error = sfnt_get_glyph_offset(pdr, pfont42, char_code, &offset0, &offset1);
+            ulong offset0;
+	    bool error = sfnt_get_glyph_offset(pdr, pfont42, char_code, &offset0);
 
-            glyph_length = (error ? -1 : offset1 - offset0);
+            glyph_length = (error ? -1 : pfont42->data.len_glyphs[char_code]);
             if (buf != 0 && !error) {
                 sfnts_reader r;
                 sfnts_reader_init(&r, pdr);
@@ -1181,7 +1183,7 @@ static int FAPI_refine_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_base *pbfont, c
 	pbfont->FontBBox.q.x = (double)BBox[2] * size1 / size;
 	pbfont->FontBBox.q.y = (double)BBox[3] * size1 / size;
 	if (dict_find_string(op, "FontBBox", &v) > 0) {
-	    if(!r_has_type(v, t_array) && !r_has_type(v, t_shortarray))
+	    if(!r_has_type(v, t_array) && !r_has_type(v, t_shortarray) && !r_has_type(v, t_mixedarray))
 		return_error(e_invalidfont);
 	    if (r_size(v) < 4)
 		return_error(e_invalidfont);
@@ -1190,7 +1192,7 @@ static int FAPI_refine_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_base *pbfont, c
 	    make_real(&mat[1], pbfont->FontBBox.p.y);
 	    make_real(&mat[2], pbfont->FontBBox.q.x);
 	    make_real(&mat[3], pbfont->FontBBox.q.y);
-	    if(r_has_type(v, t_shortarray)) {
+	    if(r_has_type(v, t_shortarray) || r_has_type(v, t_mixedarray)) {
 		/* Create a new full blown array in case the values are reals */
 		code = ialloc_ref_array(&arr, a_all, 4, "array");
 		if (code < 0)
@@ -1277,6 +1279,7 @@ static int zFAPIrebuildfont(i_ctx_t *i_ctx_p)
     const byte *pchars;
     uint len;
     font_data *pdata;
+    FAPI_server *I;
 
     if (code < 0)
 	return code;
@@ -1297,6 +1300,7 @@ static int zFAPIrebuildfont(i_ctx_t *i_ctx_p)
 	    return code;
     }
     pdata = (font_data *)pfont->client_data;
+    I = pbfont->FAPI;
     if (dict_find_string(op - 1, "Path", &v) <= 0 || !r_has_type(v, t_string))
         v = NULL;
     if (pfont->FontType == ft_CID_encrypted && v == NULL) {
@@ -1316,6 +1320,7 @@ static int zFAPIrebuildfont(i_ctx_t *i_ctx_p)
         if (v != NULL)
             font_file_path = ref_to_string(v, imemory_global, "font file path");
         code = FAPI_refine_font(i_ctx_p, op - 1, pbfont, font_file_path);
+	memcpy(&I->initial_FontMatrix, &pbfont->FontMatrix, sizeof(gs_matrix));
         if (font_file_path != NULL)
             gs_free_string(imemory_global, (byte *)font_file_path, r_size(v) + 1, "font file path");
         code1 = gs_notify_register(&pfont->notify_list, notify_remove_font, pbfont);
@@ -1608,7 +1613,7 @@ retry_oversampling:
 	I->face.HWResolution[1] != dev->HWResolution[1]
        ) {
 	FAPI_font_scale font_scale = {{1, 0, 0, 1, 0, 0}, {0, 0}, {1, 1}, true};
-        gs_matrix *base_font_matrix = &pbfont->base->orig_FontMatrix;
+        gs_matrix *base_font_matrix = &I->initial_FontMatrix;
         double dx, dy;
 
 	I->face.font_id = pbfont->id;
@@ -1753,8 +1758,33 @@ retry_oversampling:
 	       using src_type, dst_type. Should adjust the 'matrix' above.
                Call get_font_proportional_feature for proper choice.
             */
-        } else
-            cr.char_codes[0] = client_char_code;
+	} else {
+            ref *CIDMap, Str;
+	    byte *Map;
+	    int ccode = client_char_code;
+
+	    /* The PDF Reference says that we should use a CIDToGIDMap, but the PDF
+	     * interpreter converts this into a CIDMap (see pdf_font.ps, processCIDToGIDMap)
+	     */
+	    if (dict_find_string(pdr, "CIDMap", &CIDMap) > 0 && !r_has_type(CIDMap, t_name)) {
+		if (r_has_type(CIDMap, t_array)) {
+		    /* Too big for single string, so its an array of 2 strings */
+		    if (client_char_code < 32767) 
+			code = array_get(imemory, CIDMap, 0, &Str);
+		    else {
+			code = array_get(imemory, CIDMap, 1, &Str);
+			ccode -= 32767;
+		    }
+		    if (code < 0)
+		        return code;
+		    Map = &Str.value.bytes[ccode * 2];
+		} else
+		    Map = &CIDMap->value.bytes[ccode * 2];
+		cr.char_codes[0] = (Map[0] << 8) + Map[1];
+	    }
+	    else
+		cr.char_codes[0] = client_char_code;
+	}
     } else if (is_TT_from_type42) {
         /* This font must not use 'cmap', so compute glyph index from CharStrings : */
 	ref *CharStrings, *glyph_index;
@@ -1766,8 +1796,19 @@ retry_oversampling:
 		return code;
         } else if (r_has_type(glyph_index, t_integer))
             cr.char_codes[0] = glyph_index->value.intval;
-        else
-            return_error(e_invalidfont);
+	else {
+	    /* Check execution stack has space for BuldChar proc and finish_render */
+	    check_estack(2);
+	    /* check space and duplicate the glyph index for BuildChar */
+	    check_op(1);
+	    push(1);
+	    ref_assign_inline(op, op - 1);
+	    /* Come back to fapi_finish_render after running the BuildChar */
+	    push_op_estack(fapi_finish_render);
+	    ++esp;
+	    ref_assign(esp, glyph_index);
+	    return o_push_estack;
+	}
         cr.is_glyph_index = true;
     } else if (is_embedded_type1) {
         /*  Since the client passes charstring by callback using I->ff.char_data,
@@ -1988,6 +2029,7 @@ retry_oversampling:
     penum_s->fapi_glyph_shift.x = penum_s->fapi_glyph_shift.y = 0;
     if (sbw_state == SBW_FROM_RENDERER) {
 	sbw[2] = metrics.escapement / em_scale_x;
+	sbw[3] = metrics.v_escapement / em_scale_y;
 	if (pbfont->FontType == 2) {
 	    gs_font_type1 *pfont1 = (gs_font_type1 *)pbfont;
 
@@ -2051,10 +2093,15 @@ retry_oversampling:
      * from glyph code ? Currently we keep a compatibility
      * to the native GS font renderer without a deep analyzis.
      */
-    code = zchar_set_cache(i_ctx_p, pbfont, &char_name,
+    if (bCID)
+	code = zchar_set_cache(i_ctx_p, pbfont, op,
 		           NULL, sbw + 2, &char_bbox,
 			   fapi_finish_render, &exec_cont, sbw);
-    if (code >= 0 && exec_cont != 0)
+    else
+	code = zchar_set_cache(i_ctx_p, pbfont, &char_name,
+		           NULL, sbw + 2, &char_bbox,
+			   fapi_finish_render, &exec_cont, sbw);
+    if (code >= 0 && exec_cont != 0) 
 	code = (*exec_cont)(i_ctx_p);
     if (code != 0) {
         if (code < 0) {
