@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfile.c 9778 2009-06-05 05:55:54Z alexcher $ */
+/* $Id: zfile.c 10120 2009-10-04 12:42:07Z alexcher $ */
 /* Non-I/O file operators */
 #include "memory_.h"
 #include "string_.h"
@@ -968,7 +968,9 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
 
 		/* We concatenate directly since gp_file_name_combine_*
 		 * rules are not correct for other devices such as %rom% */
-		gs_parse_file_name(&pname, pstr, plen);
+		code = gs_parse_file_name(&pname, pstr, plen);
+		if (code < 0)
+		    continue;
 		memcpy(buffer, pname.fname, pname.len);
 		memcpy(buffer+pname.len, fname, flen);
 		code = pname.iodev->procs.open_file(pname.iodev, buffer, pname.len + flen, fmode,
